@@ -20,20 +20,28 @@
 #include <string.h>
 #include <assert.h>
 #include <wchar.h>
+#include <limits.h>
+
+#define MODEL_2CH_PRIVATE_DATA
 
 #include "utils/nt_std_t.h"
 #include "utils/text.h"
 #include "_2ch/model_2ch.h"
 
-static nt_category_tp get_last_category(nt_2ch_model_tp modelp);
+static nt_category_handle get_last_category(nt_2ch_model_tp modelp);
 
 
-BOOL parse_thread(nt_thread_tp threadp, const wchar_t *linep)
+BOOL parse_thread(nt_thread_handle h_thread, const wchar_t *linep)
 {
 	wchar_t buf[1024*3+1];
+	nt_res_handle h_res;
 	nt_res_tp resp;
 	wchar_t *cptr1, *cptr2, *cptr3, *cptr4;
 	wchar_t *name, *mail, *misc, *msg;
+	
+	assert(h_thread);
+	assert(h_thread->chk_sum == NT_2CH_THREAD_CHK_SUM);
+	
 	if(!linep)
 		return FALSE;
 	int len = wcslen(linep);
@@ -59,20 +67,80 @@ BOOL parse_thread(nt_thread_tp threadp, const wchar_t *linep)
 	if(cptr4)
 		*cptr4 = L'\0';
 
+	*cptr1 = *cptr2 = *cptr3 = L'\0';
 	name = buf;
 	mail = cptr1 + 2;
 	misc = cptr2 + 2;
 	msg = cptr3 + 2;
-	*cptr1 = *cptr2 = *cptr3 = L'\0';
 
-	name = buf;
-
-	resp = nt_res_alloc(threadp, name, mail, misc, msg);
-	if(!resp)
+	h_res = nt_res_alloc(h_thread, name, mail, misc, msg);
+	if(!h_res)
 		return FALSE;
-	threadp->num_res++;
-	resp->seq_no = threadp->num_res;
+	
+	nt_res_release_ref(h_res);
+	
+	resp = (nt_res_tp)h_res;
+	resp->seq_no = ((nt_thread_tp)h_thread)->num_res;
+	return TRUE;
+}
 
+BOOL parse_thread_machi_bbs(nt_thread_handle h_thread, const wchar_t *linep)
+{
+	wchar_t buf[1024*3+1];
+	nt_res_handle h_res;
+	nt_res_tp resp;
+	wchar_t *cptr1, *cptr2, *cptr3, *cptr4;
+	wchar_t *name, *mail, *misc, *msg;
+	long num;
+	
+	assert(h_thread);
+	assert(h_thread->chk_sum == NT_2CH_THREAD_CHK_SUM);
+	
+	if(!linep)
+		return FALSE;
+	int len = wcslen(linep);
+	if(len <= 0)
+		return FALSE;
+	if(len >= sizeof(buf))
+		return FALSE;
+
+	wcscpy(buf, linep);
+	cptr1 = wcsstr(buf, L"<>");
+	if(!cptr1)
+		return FALSE;
+
+	cptr2 = wcsstr(cptr1+2, L"<>");
+	if(!cptr2)
+		return FALSE;
+
+	cptr3 = wcsstr(cptr2+2, L"<>");
+	if(!cptr3)
+		return FALSE;
+
+	cptr4 = wcsstr(cptr3+2, L"<>");
+	if(!cptr4)
+		return FALSE;
+
+	*cptr1 = *cptr2 = *cptr3 = *cptr4 = L'\0';
+	num = wcstol(buf, NULL, 10);
+	if(num == 0 || num == LONG_MIN || num == LONG_MAX)
+		return FALSE;
+	name = cptr1 + 2;
+	mail = cptr2 + 2;
+	misc = cptr3 + 2;
+	msg = cptr4 + 2;
+	cptr4 = wcsstr(msg, L"<>");
+	if(cptr4)
+		*cptr4 = L'\0';
+
+	h_res = nt_res_alloc(h_thread, name, mail, misc, msg);
+	if(!h_res)
+		return FALSE;
+	
+	nt_res_release_ref(h_res);
+	
+	resp = (nt_res_tp)h_res;
+	resp->seq_no = (int)num;
 	return TRUE;
 }
 
@@ -109,25 +177,34 @@ wchar_t* parse_thread_title(const wchar_t *linep)
 }
 
 
-BOOL parse_board(nt_board_tp boardp, const wchar_t *linep)
+BOOL parse_board(nt_board_handle h_board, const wchar_t *linep)
 {
 	wchar_t buf[256+1];
-	nt_thread_tp threadp;
+	nt_thread_handle h_thread;
 	wchar_t *cptr1, *cptr2, *cptr3;
 	int len, num_res;
 
+	assert(h_board);
+	assert(h_board->chk_sum == NT_2CH_BOARD_CHK_SUM);
+	
 	len = wcslen(linep);
-	if(sizeof(buf)-1 < len)
+	if(sizeof(buf)-1 < len*sizeof(wchar_t))
 		return FALSE;
-	if(len >= sizeof(buf))
+	if(len*sizeof(wchar_t) >= sizeof(buf))
 		return FALSE;
 
 	wcscpy(buf, linep);
 	cptr1 = wcsstr(buf, L"<>");
-	if(!cptr1)
-		return FALSE;
-	*cptr1 = L'\0';
-	cptr1 += 2;
+	if(!cptr1){
+		cptr1 = wcsstr(buf, L",");
+		if(!cptr1)
+			return FALSE;
+		*cptr1 = L'\0';
+		cptr1++;
+	}else{
+		*cptr1 = L'\0';
+		cptr1 += 2;
+	}
 	cptr2 = wcsrchr(cptr1, L'(');
 	if(cptr2)
 		cptr3 = wcsrchr(cptr2, L')');
@@ -138,27 +215,30 @@ BOOL parse_board(nt_board_tp boardp, const wchar_t *linep)
 	}else{
 		num_res = 0;
 	}
-	threadp = nt_thread_alloc(
-			&boardp->threadlistp, cptr1, buf, num_res);
-	if(!threadp)
+	h_thread = nt_thread_alloc(
+			h_board, cptr1, buf, num_res);
+	if(!h_thread)
 		return FALSE;
-
+	
+	nt_thread_release_ref(h_thread);
+	
 	return TRUE;
 }
 
 
-BOOL parse_board_menu(nt_2ch_model_tp modelp,
+BOOL parse_board_menu(nt_2ch_model_handle h_model,
 		const wchar_t *linep)
 {
-
+	nt_category_handle h_category;
+	nt_board_handle h_board;
 	wchar_t name[1024*2];
 	int len;
 	wchar_t *cptr1, *cptr2, *cptr3, *cptr4;
 	wchar_t *addressp;
-	nt_category_tp categoryp;
 	int offset;
 
-	assert(modelp);
+	assert(h_model);
+	assert(h_model->chk_sum == NT_2CH_MODEL_CHK_SUM);
 	assert(linep);
 
 	cptr1 = wcsstr(linep, L"<BR><B>");
@@ -170,8 +250,10 @@ BOOL parse_board_menu(nt_2ch_model_tp modelp,
 		len = cptr2 - cptr1;
 		wcsncpy(name, cptr1, len);
 		name[len] = L'\0';
-		if(!nt_category_alloc(modelp, name))
+		h_category = nt_category_alloc(h_model, name);
+		if(!h_category)
 			return FALSE;
+		nt_category_release_ref(h_category);
 		return TRUE;
 	}
 	cptr1 = wcsstr(linep, L"<A HREF=");
@@ -201,17 +283,20 @@ BOOL parse_board_menu(nt_2ch_model_tp modelp,
 		len = cptr3 - cptr2;
 		wcsncpy(cptr4, cptr2, len);
 		cptr4[len] = L'\0';
-		categoryp = get_last_category(modelp);
-		if(!categoryp)
+		h_category = get_last_category((nt_2ch_model_tp)h_model);
+		if(!h_category)
 			return FALSE;
-		if(!nt_board_alloc(categoryp, cptr4, addressp))
+		nt_category_release_ref(h_category);
+		h_board = nt_board_alloc(h_category, cptr4, addressp);
+		if(!h_board)
 			return FALSE;
+		nt_board_release_ref(h_board);
 		return TRUE;
 	}
 	return FALSE;
 }
 
-static nt_category_tp get_last_category(nt_2ch_model_tp modelp)
+static nt_category_handle get_last_category(nt_2ch_model_tp modelp)
 {
 	nt_category_tp categoryp;
 
@@ -220,5 +305,6 @@ static nt_category_tp get_last_category(nt_2ch_model_tp modelp)
 		return NULL;
 	linkp = linkp->prev;
 	categoryp = (nt_category_tp)linkp->data;
-	return categoryp;
+	nt_category_add_ref(&categoryp->handle);
+	return &categoryp->handle;
 }
