@@ -63,7 +63,7 @@ extern BOOL nt_http_post(const char *url,
 		nt_link_tp headers, nt_cookie_tp cookiep);
 extern BOOL nt_http_get(const char *url, const char *out_path,
 		const char *referer, const char *user_agent, 
-		nt_link_tp headers, BOOL range);
+		nt_link_tp headers, BOOL range, BOOL ignore_cache);
 extern int nt_http_parse_response_header(int readfd,
 		nt_http_response_header_tp responsep,
 		nt_cookie_tp cookeip, const char *host);
