@@ -18,9 +18,12 @@
 #ifndef _NT_SOCKET_H_
 #define _NT_SOCKET_H_
 
+#include "utils/nt_mutex.h"
+
 typedef struct tag_nt_socket_t *nt_socket_tp;
 typedef struct tag_nt_socket_t{
 	struct sockaddr_in  _sockaddr_in;
+	nt_mutex_handle h_mutex;
 } nt_socket_t;
 
 extern nt_socket_tp nt_socket_init(int port, char *addr);
