@@ -38,6 +38,7 @@
 #include "usr/usr_db_t.h"
 #include "utils/nt_mutex.h"
 #include "utils/nt_pthread.h"
+#include "utils/nt_timer.h"
 #include "usr/favorite_t.h"
 #include "_2ch/_2ch.h"
 #include "_2ch/maru_2ch.h"
@@ -94,15 +95,7 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	if(!nt_2ch_model_init()){
-		fputs("Failed to read board menu data.\n", stderr);
-		goto ERROR_TRAP;
-	}
-
-	if(!nt_init_board_menu()){
-		fputs("Failed to initialize board menu data.\n", stderr);
-		goto ERROR_TRAP;
-	}
+	nt_timer_lib_init();
 	
 	nt_mutex_lib_init();
 	
@@ -114,6 +107,17 @@ int main(int argc, char *argv[])
 			fputs(err_msg, stderr);
 		goto ERROR_TRAP;
 	}
+	
+	if(!nt_2ch_model_init()){
+		fputs("Failed to read board menu data.\n", stderr);
+		goto ERROR_TRAP;
+	}
+
+	if(!nt_init_board_menu()){
+		fputs("Failed to initialize board menu data.\n", stderr);
+		goto ERROR_TRAP;
+	}
+	
 
 	h_favorite = nt_favorite_alloc(L"favorite");
 	if(!h_favorite){
@@ -183,9 +187,24 @@ ERROR_TRAP:
 	
 	nt_mutex_lib_finish();
 
+	nt_timer_lib_finish();
+	
 	return (result);
 }
 
+#define TIMER_ID_AUTO_UPDATE_NONE 0
+#define TIMER_ID_AUTO_UPDATE 1
+#define TIMER_ID_AUTO_SCROLL 2
+static int auto_update_timer_func(int id)
+{
+	switch(id){
+	case TIMER_ID_AUTO_UPDATE:
+		return TIMER_ID_AUTO_UPDATE;
+	case TIMER_ID_AUTO_SCROLL:
+		return TIMER_ID_AUTO_SCROLL;
+	}
+	return TIMER_ID_AUTO_UPDATE_NONE;
+}
 
 static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 		nt_favorite_handle h_favorite, 
@@ -216,6 +235,10 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 	void *handle;
 	nt_pthread_result_t async_data;
 	nt_link_tp linkp;
+	BOOL auto_scrolling;
+	nt_timer_handle h_timer_auto_update;
+	nt_timer_handle h_timer_auto_scroll;
+	nt_timer_handle h_timer;
 	
 	state = DISP_STATE_BOARDMENU;
 	if(INIT_DISP_STATE == NT_INTI_DISP_FAVORITE){
@@ -227,6 +250,10 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 
 	if(MARU_ID && MARU_PW) 
 		marup = nt_maru_2ch_alloc(MARU_ID, MARU_PW);
+	h_timer_auto_scroll = nt_timer_alloc(
+			TIMER_ID_AUTO_SCROLL, -1, auto_update_timer_func);
+	h_timer_auto_update = nt_timer_alloc(
+			TIMER_ID_AUTO_UPDATE, NT_AUTO_UPDATE_INTERVAL, auto_update_timer_func);
 
 	h_write_data = NULL;
 	marup = NULL;
@@ -255,6 +282,8 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 			wclear(scrp);
 		else
 			werase(scrp);
+			
+		
 		switch(disp_state){
 		case DISP_STATE_BOARDMENU:
 			werase(bwinp->wp);
@@ -397,6 +426,20 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 				goto END_WHILE;
 			rwinp->key = ch;
 			state = disp_reslist(rwinp, state, h_sel_items, db_handle);
+			auto_scrolling = FALSE;
+			if(DISP_CMD(state)){
+				if(DISP_CMD(state) == DISP_CMD_AUTO_SCROLL){
+					if(0 >= nt_timer_get_interval(h_timer_auto_scroll)){
+						nt_timer_set_interval(
+							h_timer_auto_scroll, NT_AUTO_SCROLL_INTERVAL);
+					}
+					auto_scrolling = TRUE;
+				}
+				DISP_CLR_CMD(state);
+			}
+			if(!auto_scrolling){
+				nt_timer_set_interval(h_timer_auto_scroll, -1);
+			}
 			if(DISP_STATE_ERROR == state){
 				goto END_WHILE;
 			}else if(state == DISP_STATE_REFRESH){
@@ -618,14 +661,33 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 		touchwin(scrp);
 		wrefresh(scrp);
 		
-		timeout(NT_MAINLOOP_POOLING_INTERVAL);
+		timeout(NT_MAINLOOP_POLLING_INTERVAL);
 		do{
 			ch = getch();
 			if(ch == ERR){
 				async_data = nt_pthread_get_result_from_que();
 				if(async_data.code == NT_PTHREAD_RESULT_UPDATE_BOARD){
-					
 					ch = NT_KEY_CMD_BOARD_UPDATE;
+					//fprintf(stderr, "Async result recieved.!\n");
+				}else{
+					h_timer = nt_timer_ring_a_bell();
+					if(h_timer){
+						switch(nt_timer_get_id(h_timer)){
+						case TIMER_ID_AUTO_SCROLL:
+							ch = NT_KEY_CMD_AUTO_SCROLL;
+							break;
+						case TIMER_ID_AUTO_UPDATE:
+							linkp = nt_favorite_get_update_board_list(
+									app_2ch_model, h_favorite);
+							if(linkp){
+								nt_read_board_list(linkp);
+								nt_all_link_free(linkp, _2ch_selected_item_free);
+								//fprintf(stderr, "Update request calling.!\n");
+							}
+							break;
+						}
+						nt_timer_release_ref(h_timer);
+					}
 				}
 			}
 		}while(ch == '\n' || ch == '\r' || ch == ERR);
@@ -680,6 +742,10 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 		}/* end switch*/
 	}/* end while */
 END_WHILE:
+	if(h_timer_auto_update)
+		nt_timer_release_ref(h_timer_auto_update);
+	if(h_timer_auto_scroll)
+		nt_timer_release_ref(h_timer_auto_scroll);
 	if(cookiep)
 		nt_unload_cookie(cookiep);
 	if(marup)
