@@ -31,6 +31,7 @@
 #include "utils/text.h"
 #include "utils/zip.h"
 #include "utils/db.h"
+#include "utils/nt_mutex.h"
 #include "net/nt_socket.h"
 #include "net/nt_http.h"
 #include "net/nt_cookie.h"
@@ -263,7 +264,7 @@ BOOL nt_http_get(const char *url, const char *out_path,
 		strcat(data, referer);
 	}
 	if(IS_SET_FLAG(headerp, RANGE_FLAG) && range){
-		if(0 <= sprintf(itoa_buf, "%lu-", headerp->fsize)){
+		if(0 <= sprintf(itoa_buf, "%zu-", headerp->fsize)){
 			strcat(data, "\r\nRange: bytes=");
 			strcat(data, itoa_buf);
 		}
@@ -296,6 +297,9 @@ BOOL nt_http_get(const char *url, const char *out_path,
 			nt_http_free_header(headerp);
 			return FALSE;
 		}
+		/* Not implemented yet. */
+		assert(1);
+		return FALSE;
 	}else{
 		if(headerp->port == -1)
 			headerp->port = 80;
@@ -433,6 +437,7 @@ static BOOL nt_http_init_header2(
     const char *log_path;
 	DBM *dbm;
 	struct stat st;
+	nt_mutex_handle h_mutex;
 
 	assert(headerp);
 	assert(out_path);
@@ -440,10 +445,21 @@ static BOOL nt_http_init_header2(
 	if(!nt_db_cpy_key(out_path, key)){
 		return FALSE;
 	}
-
+	
+	h_mutex = nt_mutex_get_one_time_handle(NT_MUTEX_ROOT_NAME_FILE);
+	if(!h_mutex){
+		return FALSE;
+	}
+	if(!nt_mutex_add_moniker(h_mutex, LOG_DB_W_NAME)){
+		return FALSE;
+	}
+	if(!nt_mutex_lock(h_mutex)){
+		return FALSE;
+	}
 	log_path	=	nt_db_get_log_path();
 	dbm	=	dbm_open((char*)log_path, O_RDWR | O_CREAT, 0600);
 	if(dbm == NULL){
+		nt_mutex_unlock(h_mutex);
 		return FALSE;
 	}
 
@@ -458,6 +474,7 @@ static BOOL nt_http_init_header2(
 		headerp->last_modified = NULL;
 	}
 	dbm_close(dbm);
+	nt_mutex_unlock(h_mutex);
 
 	if(0 == stat(out_path, &st)){
 		headerp->fsize = st.st_size;
@@ -465,7 +482,7 @@ static BOOL nt_http_init_header2(
 	}else{
 		headerp->fsize = -1;
 	}
-
+	
 	return TRUE;
 
 }
