@@ -33,20 +33,22 @@
 #include "ui/disp.h"
 
 
-static BOOL read_editor(FILE *fp, nt_write_data_tp writep,
+static BOOL read_editor(FILE *fp, nt_write_data_handle h_write_data,
 		char *buf_reuse, size_t buf_len);
 
-BOOL disp_editor(nt_write_data_tp writep)
+BOOL disp_editor(nt_write_data_handle h_write_data)
 {
 	FILE *tmp_fp;
 	pid_t pid;
 	char fname[512];
 	char buf[1024*2];
 	int len, ret;
+	const char *name, *mail;
+	
+	assert(h_write_data);
 
-	assert(writep);
-
-	writep->status_msg = NULL;
+	name = nt_write_data_get_name(h_write_data);
+	mail = nt_write_data_get_mail(h_write_data);
 
 	pid = getpid();
 
@@ -56,15 +58,15 @@ BOOL disp_editor(nt_write_data_tp writep)
 		return FALSE;
 	
 	strcpy(buf, "name: ");
-	if(writep->name)
-		strcat(buf, writep->name);
+	if(name)
+		strcat(buf, name);
 	strcat(buf, "\n");
 	len = strlen(buf);	
 	if(len > fwrite(buf, sizeof(char), len, tmp_fp))
 		goto ERROR_TRAP1;
 
 	sprintf(buf, "mail: %s\n\n", 
-			(writep->mail) ? writep->mail : "sage");
+			(mail) ? mail : "sage");
 	len = strlen(buf);	
 	if(len > fwrite(buf, sizeof(char), len, tmp_fp))
 		goto ERROR_TRAP1;
@@ -83,11 +85,11 @@ BOOL disp_editor(nt_write_data_tp writep)
 	if(!tmp_fp)
 		goto ERROR_TRAP2;
 
-	if(!read_editor(tmp_fp, writep, buf, sizeof(buf)))
+	if(!read_editor(tmp_fp, h_write_data, buf, sizeof(buf)))
 		goto ERROR_TRAP1;
 	
 
-	writep->status_msg = NT_INFO_WRITE_MSG_SUCCESS;
+	nt_write_data_set_status_msg(h_write_data, NT_INFO_WRITE_MSG_SUCCESS);
 	fclose(tmp_fp);
 	unlink(fname);
 	return TRUE;
@@ -99,7 +101,7 @@ ERROR_TRAP2:
 	return FALSE;
 }
 
-BOOL read_editor(FILE *fp, nt_write_data_tp writep, 
+BOOL read_editor(FILE *fp, nt_write_data_handle h_write_data, 
 		char *buf_reuse, size_t buf_len)
 {
 	char *cptr, *cptr2;
@@ -122,13 +124,15 @@ BOOL read_editor(FILE *fp, nt_write_data_tp writep,
 			continue;
 
 		if(0 == strcmp(cptr2, "name")){
-			if(writep->name)
-				free(writep->name);
-			writep->name = nt_trim(cptr+1);
+			nt_write_data_set_name(h_write_data,cptr+1);
+			//if(writep->name)
+			//	free(writep->name);
+			//writep->name = nt_trim(cptr+1);
 		}else if(0 == strcmp(cptr2, "mail")){
-			if(writep->mail)
-				free(writep->mail);
-			writep->mail = nt_trim(cptr+1);
+			nt_write_data_set_mail(h_write_data,cptr+1);
+			//if(writep->mail)
+			//	free(writep->mail);
+			//writep->mail = nt_trim(cptr+1);
 		}
 	}
 	/* Read message to send */
@@ -140,16 +144,17 @@ BOOL read_editor(FILE *fp, nt_write_data_tp writep,
 	}
 	len = strlen(buf_reuse);
 	if(len == 0){
-		writep->status_msg = NT_ERR_MSG_WRITE_MSG_LENGTH;
+		nt_write_data_set_status_msg(h_write_data, NT_ERR_MSG_WRITE_MSG_LENGTH);
 		return FALSE;
 	}
 	if(!strcmp(buf_reuse, "Please write your message here\n")){
-		writep->status_msg = NT_ERR_MSG_WRITE_MSG_ABORTED;
+		nt_write_data_set_status_msg(h_write_data, NT_ERR_MSG_WRITE_MSG_ABORTED);
 		return FALSE;
 	}
-	cptr = malloc(len+1);
-	strcpy(cptr, buf_reuse);
-	writep->msg = cptr;
+	//cptr = malloc(len+1);
+	//strcpy(cptr, );
+	//writep->msg = cptr;
+	nt_write_data_set_msg(h_write_data, buf_reuse);
 	return TRUE;
 }
 
