@@ -239,7 +239,7 @@ int disp_favorite(nt_window_tp wp,
 						result_state |= DISP_CMD_DEL_FAVORITE_THREAD;
 					}else if(sel_flag){
 						(*h_sel_threadp) = nt_searched_thread_alloc(
-							board_name, dat_name, NULL);
+							board_name, dat_name, title);
 						if((*h_sel_threadp))
 							result_state |= DISP_CMD_SEL_FAVORITE_THREAD;
 					}else if(sel_flag2){
