@@ -33,7 +33,7 @@
 #include "ui/disp.h"
 
 
-int disp_html_result(nt_write_data_tp writep)
+int disp_html_result(nt_write_data_handle h_write_data)
 {
 	FILE *tmp_fp;
 	pid_t pid;
@@ -41,10 +41,12 @@ int disp_html_result(nt_write_data_tp writep)
 	char buf[1024*2];
 	int len, len2, ret;
 	wchar_t *cptr, *sptr, *html_str;
+	const wchar_t *result_html;
 	size_t sz;
 	int result = -1;
 
-	if(!writep->result_html)
+	result_html = nt_write_data_get_html_result(h_write_data);
+	if(!result_html)
 		return -1;
 	pid = getpid();
 
@@ -54,14 +56,14 @@ int disp_html_result(nt_write_data_tp writep)
 		return -1;
 
 	
-	len = wcslen(writep->result_html);
+	len = wcslen(result_html);
 	html_str = malloc(sizeof(wchar_t)*(len+1));
 	if(!html_str){
 		fclose(tmp_fp);
 		unlink(fname);
 		return -1;
 	}
-	wcscpy(html_str, writep->result_html);
+	wcscpy(html_str, result_html);
 	cptr = wcsstr(html_str, L"書きこみました");
 	if(cptr){
 		result = 0;
