@@ -16,19 +16,161 @@
     
 */
 #include <stdlib.h>
+#include <wchar.h>
 #include <memory.h>
 #include <assert.h>
 #include "utils/nt_std_t.h"
+#include "utils/text.h"
 
 #define NT_ENUM_CHK_SUM (878428)
+#define NT_MAP_CHK_SUM (878429)
+#define NT_QUEUE_CHK_SUM (878430)
 
 typedef struct tag_nt_enum_t *nt_enum_tp;
 typedef struct tag_nt_enum_t {
 	nt_enum_handle_t handle;
 	nt_link_tp source;
 	nt_link_tp current;
+	
+	nt_memfree_fn free_func;
 }nt_enum_t;
 
+typedef struct tag_nt_map_t *nt_map_tp;
+typedef struct tag_nt_map_t {
+	nt_map_handle_t handle;
+	nt_link_tp key_value_list;
+}nt_map_t;
+
+typedef struct tag_nt_queue_t *nt_queue_tp;
+typedef struct tag_nt_queue_t{
+	nt_queue_handle_t handle;
+	nt_link_tp linkp;
+} nt_queue_t;
+
+
+nt_map_handle nt_map_alloc()
+{
+	nt_map_tp mapp;
+	mapp = malloc(sizeof(nt_map_t));
+	if(!mapp)
+		return NULL;
+	
+	mapp->handle.chk_sum = NT_MAP_CHK_SUM;
+	mapp->key_value_list = NULL;
+	return &mapp->handle;
+}
+
+void nt_map_free(nt_map_handle handle, nt_memfree_fn value_free_func)
+{
+	nt_map_tp mapp;
+	nt_link_tp linkp, nextp;
+	nt_w_key_value_tp kvp;
+	assert(handle);
+	assert(handle->chk_sum == NT_MAP_CHK_SUM);
+	mapp = (nt_map_tp)handle;
+	if(mapp->key_value_list){
+		linkp = mapp->key_value_list;
+		do{
+			nextp = linkp->next;
+			kvp = (nt_w_key_value_tp)linkp->data;
+			if(kvp->value)
+				if(value_free_func){
+					(value_free_func)(kvp->value);
+			}
+			free(kvp->key);
+			free(kvp);
+			free(linkp);
+			linkp = nextp;
+		}while(linkp != mapp->key_value_list);
+	}
+	free(mapp);
+}
+
+BOOL nt_map_add_pair(nt_map_handle handle, const wchar_t *key, void *value)
+{
+	nt_map_tp mapp;
+	nt_link_tp linkp;
+	nt_w_key_value_tp kvp;
+	assert(handle);
+	assert(handle->chk_sum == NT_MAP_CHK_SUM);
+	mapp = (nt_map_tp)handle;
+	kvp = nt_map_find(handle, key);
+	assert(!kvp);
+	kvp = nt_w_key_value_alloc(key, value);
+	if(!kvp)
+		return FALSE;
+	linkp = nt_link_add_data(mapp->key_value_list, kvp);
+	if(!linkp){
+		free(kvp->key);
+		free(kvp);
+		return FALSE;
+	}
+	if(!mapp->key_value_list)
+		mapp->key_value_list = linkp;
+	return TRUE;
+}
+
+void* nt_map_find(nt_map_handle handle, const wchar_t *key)
+{
+	nt_map_tp mapp;
+	nt_link_tp linkp;
+	nt_w_key_value_tp kvp;
+	assert(handle);
+	assert(handle->chk_sum == NT_MAP_CHK_SUM);
+	mapp = (nt_map_tp)handle;
+	if(mapp->key_value_list){
+		linkp = mapp->key_value_list;
+		do{
+			kvp = (nt_w_key_value_tp)linkp->data;
+			if(0 == wcscmp(key, kvp->key))
+				return kvp->value;
+			
+			linkp = linkp->next;
+		}while(linkp != mapp->key_value_list);
+	}
+	return NULL;
+}
+
+void* nt_map_remove(nt_map_handle handle, const wchar_t *key)
+{
+	nt_map_tp mapp;
+	nt_link_tp linkp;
+	nt_w_key_value_tp kvp;
+	void *data;
+	assert(handle);
+	assert(handle->chk_sum == NT_MAP_CHK_SUM);
+	mapp = (nt_map_tp)handle;
+	if(mapp->key_value_list){
+		linkp = mapp->key_value_list;
+		do{
+			kvp = (nt_w_key_value_tp)linkp->data;
+			if(0 == wcscmp(key, kvp->key)){
+				data = kvp->value;
+				mapp->key_value_list = nt_link_remove2(
+							mapp->key_value_list, linkp);
+				free(linkp);
+				free(kvp->key);
+				free(kvp);
+				return data;
+			}
+			linkp = linkp->next;
+		}while(linkp != mapp->key_value_list);
+	}
+	return NULL;
+}
+
+int nt_map_get_count(nt_enum_handle handle)
+{
+	nt_map_tp mapp;
+	assert(handle);
+	assert(handle->chk_sum == NT_MAP_CHK_SUM);
+	mapp = (nt_map_tp)handle;
+	if(!mapp->key_value_list)
+		return 0;
+	return nt_link_num(mapp->key_value_list);
+}
+
+
 nt_enum_handle nt_enum_set(nt_link_tp linkp)
 {
 	nt_enum_tp enump;
@@ -40,9 +182,20 @@ nt_enum_handle nt_enum_set(nt_link_tp linkp)
 	
 	enump->source = linkp;
 	enump->current = NULL;
+	enump->free_func = NULL;
 	return (nt_enum_handle)enump;
 }
 
+void nt_enum_set_free_func(nt_enum_handle handle, nt_memfree_fn free_func)
+{
+	nt_enum_tp enump;
+	enump = (nt_enum_tp)handle;
+	assert(enump);
+	assert(enump->handle.chk_sum == NT_ENUM_CHK_SUM);
+	enump->free_func = 	free_func;
+}
+
+
 void nt_enum_reset(nt_enum_handle handle)
 {
 	nt_enum_tp enump;
@@ -58,6 +211,9 @@ void nt_enum_unset(nt_enum_handle handle)
 	enump = (nt_enum_tp)handle;
 	assert(enump);
 	assert(enump->handle.chk_sum == NT_ENUM_CHK_SUM);
+	if(enump->source && enump->free_func){
+		nt_all_link_free(enump->source, enump->free_func);
+	}
 	free(enump);
 }
 
@@ -368,6 +524,20 @@ nt_key_value_tp nt_key_value_alloc(char *key, char *value)
 	return ptr;
 }
 
+nt_w_key_value_tp nt_w_key_value_alloc(const wchar_t *key, void *value)
+{
+	nt_w_key_value_tp ptr = malloc(sizeof(nt_w_key_value_t));
+	if(ptr == NULL)
+		return NULL;
+	ptr->key = nt_w_str_clone(key);
+	if(!ptr->key){
+		free(ptr);
+		return NULL;
+	}
+	ptr->value = value;
+	return ptr;
+}
+
 void* nt_stack_add_last(nt_stack_tp stackp, void* data)
 {
 	int cursor;
@@ -548,70 +718,89 @@ void nt_stack_free(nt_stack_tp ptr, nt_memfree_fn free_func)
 	free(ptr);	
 }
 
-nt_queue_tp nt_queue_alloc()
+nt_queue_handle nt_queue_alloc()
 {
 	nt_queue_tp ptr;
 	ptr = malloc(sizeof(nt_queue_t));
 	if(!ptr)
 		return NULL;
-	ptr->num = 0;
 	ptr->linkp = NULL;
-	return ptr;
+	ptr->handle.chk_sum = NT_QUEUE_CHK_SUM;
+	return &ptr->handle;
 }
-void nt_queue_free(nt_queue_tp ptr, nt_memfree_fn free_func)
+void nt_queue_free(nt_queue_handle handle, nt_memfree_fn free_func)
 {
-	assert(ptr);
-	if(ptr->linkp){
-		nt_all_link_free(ptr->linkp, free_func);
+	nt_queue_tp queuep;
+	
+	assert(handle);
+	assert(handle->chk_sum == NT_QUEUE_CHK_SUM);
+	queuep = (nt_queue_tp)handle;
+	if(queuep->linkp){
+		nt_all_link_free(queuep->linkp, free_func);
 	}
-	free(ptr);
+	free(queuep);
 }
-BOOL nt_queue_push(nt_queue_tp ptr, void* data)
+
+BOOL nt_queue_push(nt_queue_handle handle, void* data)
 {
 	nt_link_tp linkp;
+	nt_queue_tp queuep;
 	
-	assert(ptr);
+	assert(handle);
+	assert(handle->chk_sum == NT_QUEUE_CHK_SUM);
+	queuep = (nt_queue_tp)handle;
 	
-	linkp = nt_link_add_data(ptr->linkp, data);
+	linkp = nt_link_add_data(queuep->linkp, data);
 	if(!linkp)
 		return FALSE;
 	
-	if(!ptr->linkp)
-		ptr->linkp = linkp;
-	ptr->num++;
+	if(!queuep->linkp)
+		queuep->linkp = linkp;
 	return TRUE;
 }
-void* nt_queue_shift(nt_queue_tp ptr)
+void* nt_queue_shift(nt_queue_handle handle)
 {
-	nt_link_tp linkp, new_headp;
-	void *datap;
-	assert(ptr);
-	assert(ptr->num >= 0);
-	if(ptr->num == 0)
+	nt_link_tp linkp, prev, next;
+	nt_queue_tp queuep;
+	void *data;
+	
+	assert(handle);
+	assert(handle->chk_sum == NT_QUEUE_CHK_SUM);
+	queuep = (nt_queue_tp)handle;
+	
+	if(!queuep->linkp)
 		return NULL;
-	assert(ptr->linkp);
 	
-	ptr->num--;
-	
-	linkp = ptr->linkp;
+	linkp = queuep->linkp;
 	if(linkp == linkp->next){
-		ptr->linkp = NULL;
-		datap = linkp->data;
+		data = linkp->data;
 		free(linkp);
-		return datap;;
+		queuep->linkp = NULL;
+		return data;
 	}
-	new_headp = linkp->next;
-	new_headp->prev = linkp->prev;
-	linkp->prev->next = new_headp;
-	ptr->linkp = new_headp;
-	datap = linkp->data;
+	
+	prev = linkp->prev;
+	next = linkp->next;
+	
+	prev->next = next;
+	next->prev = prev;
+	queuep->linkp = next;
+	data = linkp->data;
 	free(linkp);
-	return datap;
+	return data;
 }
-int nt_queue_get_count(nt_queue_tp ptr)
+
+int nt_queue_get_count(nt_queue_handle handle)
 {
-	assert(ptr);
-	return ptr->num;
+	nt_queue_tp queuep;
+	
+	assert(handle);
+	assert(handle->chk_sum == NT_QUEUE_CHK_SUM);
+	queuep = (nt_queue_tp)handle;
+	
+	if(!queuep->linkp)
+		return 0;
+	return nt_link_num(queuep->linkp);
 }
 
 
